package CPU.instructions.ADDQ;
import CPU.DisassembledOperand;
import CPU.InstructionSet;
import CPU.DisassembledInstruction;
import CPU.Size;

/**
 * The ADDQ instruction where destination is An.
 * This affects the entire reg and no condition codes are changed.
 
 * @author and copyright (c) 2013-2021 Wolfgang Lenerz
 * 
 *  0101ddd1ssmmmrrr
 *  
 *  where   ddd= data (0=8)
 *          ss= size (00=byte, 01 = word, 10 = long )
 *          mmm = ea mode =001, 
 *          rrr = ea reg
 * 
 * @version 1.01    no longer use for.. if pattern when registering, addq 8 is not shown as ...0.
 * 
 * Based on code by Tony Headford, see his licence in accompanying file.
 */
public class ADDQ_An implements InstructionSet
{
    @Override
    public final void register(final CPU.MC68000Cpu cpu2)
    {
        doRegister (0x5048,Size.Word,cpu2);
        doRegister (0x5088,Size.Long,cpu2);
    }
    
    private void doRegister(final int base, final Size size, final CPU.MC68000Cpu cpu2)
    {
        CPU.Instruction i = new CPU.Instruction()
        {
            @Override
            public final void execute(int opcode,CPU.MC68000Cpu cpu)
            { 
                int s=((opcode >> 9) & 0x07);
                if (s==0)
                    s=8;
                cpu.addr_regs[opcode & 0x07]+=s;
            }
            @Override
            public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
            {
                return disassembleOp(address, opcode, size, cpu);
            }
              
        };
        
        for (int ea_reg = 0; ea_reg < 8; ea_reg++)              // do for all An regs ...        
        {
            for (int r = 0; r < 8; r++)                         // ... and values 1-8  
            {
                cpu2.addInstruction(base + (r << 9) + ea_reg, i);
            }
        }
    }
    
    protected final DisassembledInstruction disassembleOp(int address, int opcode, Size sz,CPU.MC68000Cpu cpu)
    {
        int s = (opcode >> 9) & 0x07 ;
        if (s==0)
            s=8;
        DisassembledOperand src = new DisassembledOperand("#" + s);
        DisassembledOperand dst = cpu.disassembleDstEA(address + 2, (opcode >> 3) & 0x07, (opcode & 0x07), sz);
        return new DisassembledInstruction(address, opcode, "addq" + sz.ext(), src, dst);
    }
}
