package CPU.instructions;
import CPU.DisassembledOperand;
import CPU.InstructionSet;
import CPU.DisassembledInstruction;

/**
 * The BRA instruction in all of its variants.
 * 
 * @author and Copyright (c) for my code Wolfgang Lenerz 2013-2014
 * 
 * Based on code by Tony Headford, see his licence in accompanying file.
 * 
 */
public class BRA implements InstructionSet
{
    @Override
    public final void register(final CPU.MC68000Cpu cpu2)
    {
        CPU.Instruction i;
        
        int base = 0x6000;
        
        CPU.Instruction ib; 
        ib = new CPU.Instruction() 
        {
            
            
            @Override
            public final void execute(int opcode,CPU.MC68000Cpu cpu)
            {
                int dis = (opcode & 0xff);
                if ((dis&0x80) !=0)
                    dis|=0xffffff00;
                cpu.pc_reg +=(dis/2);
            }
            
            @Override
            public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
            {
                return disassembleOp(address, opcode,cpu);
            }
        };
      
        CPU.Instruction iw = new CPU.Instruction() 
        { 
            
            
            @Override
            public final void execute(int opcode,CPU.MC68000Cpu cpu)
            {
                cpu.pc_reg +=(cpu.readMemoryWordPCSignedInc()/2);
            }
            
            @Override
            public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
            {
                    return disassembleOp(address, opcode,cpu);
            }
           
        };
        cpu2.addInstruction(base, iw);
        
        for(int dis = 1; dis < 256; dis++)
        {
            cpu2.addInstruction(base + dis, ib);
        }
    }


    protected final DisassembledInstruction disassembleOp(int address, int opcode,CPU.MC68000Cpu cpu)
    {
            DisassembledOperand op;

            int cc = (opcode >> 8) & 0x0f;
            int dis = opcode & 0xff;
            if ((dis&0x80) !=0)
                dis|=0xffffff00;
            String name;

            if(dis != 0)
            {
                    op = new DisassembledOperand(String.format("$%08x", dis + address + 2));
                    name =  "bra.s";
            }
            else
            {
                    //word displacement
                    dis = cpu.readMemoryWordSigned(address + 2);
                    op = new DisassembledOperand(String.format("$%08x", dis + address + 2), 2, dis);
                    name = "bra.w";
            }

            return new DisassembledInstruction(address, opcode, name, op);
    }
}
