package CPU.instructions.MOVE;

/**
 * MOVE where source is absolute .W and destination is-(An).
  00ssxxxyyymmmrrr
 * s = size bit
 * x=dest register 
 * y = destination mode 100
 * m =source mode 111
 * r= source register 000
 * @author and copyright for my code (c) Wolfgang Lenerz 2013 - 2014.
 * 
 * Based on code by Tony Headford, see his licence in accompanying filelf
 */
import CPU.DisassembledOperand;
import CPU.InstructionSet;
import CPU.DisassembledInstruction;
import CPU.Size;

public class MOVEAnW4 implements InstructionSet
{
    @Override
    public final void register(final CPU.MC68000Cpu cpu2)
    {
        int base;
        CPU.Instruction i;
        for(int sz = 0; sz < 3; sz++)
        {
            if(sz == 0)
            {
                // move byte
                base = 0x1138;
                i = new CPU.Instruction()
                {
                    @Override
                    public final void execute(int opcode,CPU.MC68000Cpu cpu)
                    {
                        int s = cpu.readMemoryWordPCSignedInc();         // get mem sign extended
                        s=cpu.readMemoryByte(s);
                        int ar=(opcode>>9)&7;                               // address reg nbr
                        if (ar==7)
                        {
                            cpu.addr_regs[ar]-=2;
                        }
                        else
                        {
                            cpu.addr_regs[ar]--;
                        }
                        cpu.writeMemoryByte(cpu.addr_regs[ar], s);
                        cpu.reg_sr&=0xfff0;
                        if (s==0)
                        {
                            cpu.reg_sr+=4;
                        }
                        else if ((s&0x80)!=0)
                        {
                            cpu.reg_sr+=8;
                        }
                    }
                    @Override
                    public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                    {
                        return disassembleOp(address, opcode, Size.Byte, cpu);
                    }
                };
            }
            else if(sz == 1)
            {
                // move word
                base = 0x3138;
                i = new CPU.Instruction()
                {
                    
                    
                    @Override
                    public final void execute(int opcode,CPU.MC68000Cpu cpu)
                    {
                        int s = cpu.readMemoryWordPCSignedInc();         // get mem sign extended
                        s=cpu.readMemoryWord(s);
                        int ar=(opcode>>9)&7;           // address reg nbr
                        cpu.addr_regs[ar]-=2;
                        cpu.writeMemoryWord(cpu.addr_regs[ar], s);
                        cpu.reg_sr&=0xfff0;
                        if (s==0)
                        {
                            cpu.reg_sr+=4;
                        }
                        else if ((s&0x8000)!=0)
                        {
                            cpu.reg_sr+=8;
                        }
                    }
                    @Override
                    public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                    {
                        return disassembleOp(address, opcode, Size.Word, cpu);
                    }
                };
            }
            else
            {
                // move long
                base = 0x2138;
                i = new CPU.Instruction()
                {
                    
                    
                    @Override
                    public final void execute(int opcode,CPU.MC68000Cpu cpu)
                    {
                        int s = cpu.readMemoryWordPCSignedInc();         // get mem sign extended
                        s=cpu.readMemoryLong(s);
                        int ar=(opcode>>9)&7;           // address reg nbr
                        cpu.addr_regs[ar]-=4;
                        cpu.writeMemoryLong(cpu.addr_regs[ar], s);
                        cpu.reg_sr&=0xfff0;
                        if (s==0)
                        {
                            cpu.reg_sr+=4;
                        }
                        else if (s<0)
                        {
                            cpu.reg_sr+=8;
                        }
                    }
                    @Override
                    public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                    {
                        return disassembleOp(address, opcode, Size.Long, cpu);
                    }
                };
            }
            for(int dea_reg = 0; dea_reg < 8; dea_reg++)
            {
                cpu2.addInstruction(base + (dea_reg<<9), i);
            }
        }
    }

    protected final DisassembledInstruction disassembleOp(int address, int opcode, Size sz, CPU.MC68000Cpu cpu)
    {
        DisassembledOperand src = cpu.disassembleSrcEA(address + 2, (opcode >> 3) & 0x07, (opcode & 0x07), sz);
        DisassembledOperand dst = cpu.disassembleDstEA(address + 2 + src.bytes, (opcode >> 6) & 0x07, (opcode >> 9) & 0x07, sz);
        return new DisassembledInstruction(address, opcode, "move" + sz.ext(), src, dst);
    }
}
