package CPU.instructions.MOVE;

/**
 * MOVE where source is D16(Pc) and destination is .L.
 * 
  00ssxxxyyymmmrrr
 * s = size bit
 * x=dest register 001
 * y = destination mode 111
 * m =source mode 111
 * r= source register 010
 * 
 * @author and copyright for my code (c) Wolfgang Lenerz 2013 - 2014.
 * 
 * Based on code by Tony Headford, see his licence in accompanying filelf
 */
import CPU.DisassembledOperand;
import CPU.InstructionSet;
import CPU.DisassembledInstruction;
import CPU.Size;

public class MOVEAnY8 implements InstructionSet
{
    @Override
    public final void register(final CPU.MC68000Cpu cpu2)
    {
        int base;
        CPU.Instruction i;
        for(int sz = 0; sz < 3; sz++)
        {
            if(sz == 0)
            {
                // move byte
                base = 0x13fa;
                i = new CPU.Instruction()
                {
                    @Override
                    public final void execute(int opcode,CPU.MC68000Cpu cpu)
                    {
                        int s=(cpu.readMemoryByte(cpu.pc_reg*2 + cpu.readMemoryWordPCSignedInc()));// the first readsigned is faster than a read word and we have to cut off anyway
                        cpu.writeMemoryByte(cpu.readMemoryLongPC(), s);
                        cpu.pc_reg+=2;
                        cpu.reg_sr&=0xfff0;
                        if (s==0)
                        {
                            cpu.reg_sr+=4;
                        }
                        else if ((s&0x80)!=0)
                        {
                            cpu.reg_sr+=8;
                        }
                    }
                    @Override
                    public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                    {
                        return disassembleOp(address, opcode, Size.Byte, cpu);
                    }
                };
            }
            else if(sz == 1)
            {
                // move word
                base = 0x33fa;
                i = new CPU.Instruction()
                {
                    @Override
                    public final void execute(int opcode,CPU.MC68000Cpu cpu)
                    {
                        short s=cpu.readMemoryShort(cpu.pc_reg*2 + cpu.readMemoryWordPCSignedInc());
                        cpu.writeMemoryShort(cpu.readMemoryLongPC(), s);
                        cpu.pc_reg+=2;
                        cpu.reg_sr&=0xfff0;
                        if (s==0)
                        {
                            cpu.reg_sr+=4;
                        }
                        else if (s<0)
                        {
                            cpu.reg_sr+=8;
                        }
                    }
                   @Override
                    public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                    {
                        return disassembleOp(address, opcode, Size.Word, cpu);
                    }
                };
            }
            else
            {
                // move long
                base = 0x23fa;
                i = new CPU.Instruction()
                {
                    @Override
                    public final void execute(int opcode,CPU.MC68000Cpu cpu)
                    {
                        int s = cpu.readMemoryLong(cpu.pc_reg*2 + cpu.readMemoryWordPCSignedInc());
                        cpu.writeMemoryLong(cpu.readMemoryLongPC(), s);
                        cpu.pc_reg+=2;
                        cpu.reg_sr&=0xfff0;
                        if (s==0)
                        {
                            cpu.reg_sr+=4;
                        }
                        else if (s<0)
                        {
                            cpu.reg_sr+=8;
                        }
                    }
                    @Override
                    public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                    {
                        return disassembleOp(address, opcode, Size.Long, cpu);
                    }
                };
            }
            
            cpu2.addInstruction(base , i);
        }
    }

    protected final DisassembledInstruction disassembleOp(int address, int opcode, Size sz, CPU.MC68000Cpu cpu)
    {
        DisassembledOperand src = cpu.disassembleSrcEA(address + 2, (opcode >> 3) & 0x07, (opcode & 0x07), sz);
        DisassembledOperand dst = cpu.disassembleDstEA(address + 2 + src.bytes, (opcode >> 6) & 0x07, (opcode >> 9) & 0x07, sz);
        return new DisassembledInstruction(address, opcode, "move" + sz.ext(), src, dst);
    }
}
