package drivers;

/**
 * This tries to resolve a filepath which my contain "./", "¤/","~/" or (several of) "../", as well as "/" or "\" as separator.
 * 
 * @author and Copyright (C) Wolfgang Lenerz 2021.
 * @version 1.00 initial
 */
public class ResolvePath
{
    public static String INIFILE_DIR;
    
    public ResolvePath(String f)
    {
        ResolvePath.INIFILE_DIR=f;
    }
    
    /**
     * Gets the canonical path of a file:
     * First convert all "/" or "\" to the file separator used in this JVM, then get the canonical path of the file.Canonical path here means a path where all path abbreviations (such as "../") are resolved.
     * 
     * @param pName         pathname of a file
     * @param addSep        flag whether a path separator should be added at end (if true: add, else remove it if it is there)
     * 
     * @return the canonicial path (where all path abbreviations are resolved) or "" if any problem (i.e. null, empty or invalid path name passed as parameter).
     */
  public static String resolvePath(String pName,boolean addSep)
    {
        if (pName== null || pName.isEmpty())
            return "";
        String pathName =pName;
        boolean earlyReturn=false;
        if (pathName.startsWith("¤/"))
        {
            pathName=INIFILE_DIR+pathName.substring(2);
            earlyReturn=true;
        }
        if (pathName.startsWith("~/"))
        {
            pathName= System.getProperty("user.home")+java.io.File.separator+pathName.substring(2);
            earlyReturn=true;
        }
        pathName=pathName.replace("/", java.io.File.separator);
        pathName=pathName.replace("\\", java.io.File.separator);
        if (addSep)
        { 
            if (!pathName.endsWith(java.io.File.separator))
                pathName+=java.io.File.separator;               // name must end with separator
        }
        else
        {
             if (pathName.endsWith(java.io.File.separator))
                pathName= pathName.substring(0,pathName.length());// name must NOT end with separator
        }
        if (earlyReturn)
            return pathName;                                    // premature exit !
       
        java.io.File f=new java.io.File (pathName);
        try
        {
            pathName= f.getCanonicalPath();
            if (pathName== null || pathName.isEmpty())
                return "";
            if (addSep && !pathName.endsWith(java.io.File.separator))
                return pathName+java.io.File.separator;
            else
                return pathName;
        }
        catch (Exception e)
        {
            // NOP, fall through.
        }
        return"";                                               // this is returned if f is null or exception occurred.
    }
  
    /**
     * Resolve all pathnames in the array passed as parameter.
     * 
     * @param pName         array with pathnames to solve
     * @param addSep        flag whether a path separator should be added (if true: add)
     * @return 
     */
    public static String[] resolvePaths(String[] pName,boolean addSep)
    {
        if (pName==null)
            return null;
        int leng=pName.length;
        if (leng==0)
            return null;
        String[]resolvedNames =new String[leng];
        for (int i=0;i<leng;i++)
        {
            resolvedNames[i]=resolvePath(pName[i],addSep);
        }
        return resolvedNames;
    }
}
