package drivers.nxfa;

import smsqmulator.Types;

/**
 *
 * The Native File Access device driver class that allows SMSQE to access native files.
 * It implements 8 "drives" (NFA1_ to NFA8_) which point to native dirs.
 * 
 * <p>
 * The driver opens NFAFiles, one for each SMSQE file. Each Nfafile gets a unique number (integer 0 - 0xffff) which gets put into the
 * SMSQE channel definition block (offset 0x1E).
 * For each drive, the driver maintains a hashmap ‹Integer,NfaFile› so that it can find the Nfafile with the integer.
 * 
 * @see drivers.DeviceDriver DeviceDriver for more information on device drivers.
 * @see drivers.nxfa.XfaDriver XfaDriver of which this is an extension.
 * @author and copyright (c) 2012 Wolfgang Lenerz
 * @version v. 0.00 initial version
 * 1.01 may change useXTcc on the fly
 * 1.00 possibly use XTcc.
 */ 
public class NfaDriver extends XfaDriver
{
    /**
     * Creates this object.
     * 
     * @param cpu the smsqmulator.cpu.MC68000Cpu used.
     * @param useXTcc if True, executables files are saved as XTcc files
     */
    public NfaDriver(CPU.MC68000Cpu cpu, boolean useXTcc)
    {                      
        super(cpu,useXTcc);
        this.deviceID=Types.NFADriver;                           //'NFA0';
        this.usage=this.deviceID;
    }
    
    public void setXTcc(boolean useXTcc)
    {
       this.useXTcc = useXTcc;
    }
}
