package drivers.win;

/**
 * This is a mostly unused class for using the SMSQ/E win driver.
 * 
 * My "normal" windriver is faster, so I'm keeping that.
 * @author and copyright (c) 2014-2020 Wolfgang Lenerz
 * @version
 * 1.01 don't use cpu as field.
 * 1.00 initial version (based on the floppy disk driver)
 */
public class SWinDriver
{
    private final java.io.RandomAccessFile[] raFile;            // every access to this drive goes through this file...
    private final java.nio.channels.FileChannel [] ioChannel;   // ... and its associated channel
    private final String [] nativeFileNames;                         // names of the native dirs - an element shold never be empty - set it to null instead
    private static final int QLWA=0x514c5741;                    // 'QLWA'
    private int usage=smsqmulator.Types.WINDriver;               // usage name for device.
    private final boolean []readOnly;
    private final java.nio.channels.FileLock []flock;
    private int sectorSize = 512;
    private final boolean []isRemovable = new boolean [8];
    
    /**
     * Creates the object.
     */
    public SWinDriver()
    {
        this.raFile= new java.io.RandomAccessFile[8];
        this.ioChannel= new java.nio.channels.FileChannel [8];
        this.nativeFileNames=new String[8];
        this.readOnly=new boolean[8];
        this.flock=new java.nio.channels.FileLock[8];
        for (int i =0;i<8;i++)
            this.isRemovable[i]=false;
    }
    
    // win_drive : unlock old drive, set name
    // dir : checkrdy, wp, rs0,rs0, lock, rs0,unlock
     /**
     * Handles the required actions by calling the corresponding action routines.
     * The action routines set D0 to 0 (no error) or error number (negative), or 0x12345678 = special return. This method then
     * sets the CPU flags according to that error returen, except if it is the special return : don't touch cpu flags then, but clear D0.
     * This method also checks that the drive for which the action is to be done (in D7) is a valid drive. If not, error returns are in D0 and
     * CPY Z flag is cleared.
     * 
     * @param action what is to be done.
     */
    public void handleAction(int action,CPU.MC68000Cpu cpu)
    {
        int drive = (cpu.data_regs[7]&0xffff)-1;
        if (!driveIsOK( drive,action,cpu))
            return;
        switch (action)
        {                   // check drv ready, check wp, rd sect*2, lock , rs, unlock
            case 0 :
                readSector(drive,cpu);
                break;
            case 1:
                writeSector(drive,cpu);
                break;
            case 2:
                formatDrive(drive,cpu);
                break;
            case 3:
                checkWriteProtect(drive,cpu);
                break;
            case 4:                                             // un-lock drive
                closeRAFile(drive);
                break;
            case 5:                                             // lock drive
                lockDrive(drive,cpu);
                break;
            case 6:                                             // check drive ready
                checkAndOpenDrive(drive,cpu);
                break;
            case 7:                                             // set name of native file
                setName(drive,cpu);
                 break;
            case 8 :                                            // get name of native file
                getName(drive,cpu);
                break;
            case 9 :                                            // set removable status
                this.isRemovable[drive]=cpu.data_regs[1]!=0;    // D1 = 0 means drive is not removable
                break;
            case 10:
                cpu.data_regs[1]=this.isRemovable[drive]?-1:0;
                break;
            case 11:
                this.sectorSize=cpu.data_regs[7]&0xffff;
                break;
            case 12:
                cpu.data_regs[0]=0;
                switch (this.sectorSize)
                {
                    case 4096:
                        cpu.data_regs[5]=5;
                        break;
                    default:
                       cpu.data_regs[5]=2; 
                       break;
                }
                break;
            default :
                cpu.data_regs[0]=smsqmulator.Types.ERR_NIMP;
                break;
        }
        if (cpu.data_regs[0]!=smsqmulator.Types.SPECIAL_CLOSE_CASE)
            cpu.setZStatusFlag();
        else
            cpu.data_regs[0]=0;
    }
    
    /**
     * Checks that a drive is OK (can be read/written).
     * Sets error return depending on the action.
     * 
     * @param drive     what drive 
     * @param action    the action supposed to be made with htis drive
     * 
     * @return true if drive is OK
     */
    private boolean driveIsOK (int drive, int action,CPU.MC68000Cpu cpu)
    {
        if (drive<0 || drive > 7)                               // drive must be 0 ... 7
        {
            switch (action)
            {
                case 0:
                case 1:
                case 3:
                case 6:
                    cpu.data_regs[0]=smsqmulator.Types.ERR_MCHK;
                    break;
                case 2:
                    cpu.data_regs[0]=smsqmulator.Types.ERR_FMTF;
                    break;
                case 7:
                case 8:
                    cpu.data_regs[0]=smsqmulator.Types.ERR_IPAR;
                    break;
            }
            cpu.reg_sr&=~4;                                // remove Z flag = shows that an error occurred
            return false;
        }    
        return true;
    }
   
    /**
     * Checks whether the drive is read only.
     */
    public void checkWriteProtect(int drive,CPU.MC68000Cpu cpu)
    {
        if (this.readOnly[drive])
            cpu.reg_sr&=~4;                                // unset Z flag, drive is read only
        else
            cpu.reg_sr|=4;                                 // set Z flag, drive is RW
        cpu.data_regs[0]=0x12345678;                       // special return, don't touch cpu flags
    }
    
    /**
     * Reads a sector.
     * d0 cr   sector to read / error code 'IS On StACK
       d2 cp   number of sectors to read
       d7 c  p drive ID / number
       a1 c  p address to read into
       a3 c  p linkage block
       a4 c  p drive definition

       status return 0 or ERR.MCHK
       * 
     * the prblm is that if I lock the file hannel for a certain file
     * @param drive what drive (0...7)
    */
    public void readSector(int drive,CPU.MC68000Cpu cpu)
    {
        int sector=cpu.readMemoryLong(cpu.addr_regs[7]);
        if (this.flock[drive]==null && !this.readOnly[drive]&& sector!=0)
        {
            cpu.data_regs[0]=smsqmulator.Types.ERR_FDIU;  // cna't read /write if I don't have a lock
             return;
        }
            
            /*
            When I lock a drive from smsq/e, the iochannel corresponding to the native file for that channel is locked (lockDrive() below).
            If two drives point to the same native file (stupid, I know), SMSQE expects that one can't read any sector from
            the second drive, as long as the first drive remains locked.
                
            However, the JVM can't know that I want to access another channel on the same file for a different SMSQ/E drive, all it know is that I want to
            access a channel I have a lock o, which of course, is OK

*/
        int nbr=cpu.data_regs[2] & 0xffff;
        long position=sector * sectorSize;
        try
        { 
            this.ioChannel[drive].position(position);
            cpu.readFromFile(cpu.addr_regs[1], nbr*sectorSize, this.ioChannel[drive]);
            cpu.data_regs[0]=0;
        }
        catch (Exception e)
        {
            cpu.data_regs[0]=smsqmulator.Types.ERR_MCHK;
        }
    }
    
    /**
     * Writes a sector.
;       d0 cr   sector to read / error code 'IS On StACK
;       d2 cp   number of sectors to read
;       d7 c  p drive ID / number
;       a1 c  p address to read into
;       a3 c  p linkage block
;       a4 c  p drive definition
;
;       status return 0 or ERR.MCHK
* 
     * @param drive what drive (0...7)
     */
 /*       if (this.raFile[drive]==null)
        {
            cpu.data_regs[0]=(action ==2)?smsqmulator.Types.ERR_FMTF:smsqmulator.Types.ERR_MCHK;
            return false;
        }
        */
    public void writeSector(int drive,CPU.MC68000Cpu cpu)
    {
        int sector=cpu.readMemoryLong(cpu.addr_regs[7]);
        int nbr=cpu.data_regs[2] &0xffff;
       
        long position=sector * sectorSize;
        try
        {
            this.ioChannel[drive].position(position);
            cpu.writeToFile(cpu.addr_regs[1], nbr*sectorSize, this.ioChannel[drive]);
            cpu.data_regs[0]=0;
        }
        catch (Exception e)
        {
            cpu.data_regs[0]=smsqmulator.Types.ERR_MCHK;
        }
    }
    
    /**
     * This routine "formats" a drive in the sense that it just creates the container fiole on the local filesystem.;
;       d0 cr   format type / error code
;       d1 cr   size wished in MB / good sectors
;       d2  r   total sectors
;       a3 c  p linkage block
;       a4 c  p drive definition
 
 
 D1 = size in MB
     * 
     * @param drive what drive (0...7)
     * @param cpu
     */
    public void formatDrive(final int drive,final CPU.MC68000Cpu cpu)
    {
        long maax=Integer.MAX_VALUE;
        long length =1024*1024;
        int A0 = cpu.addr_regs[0];
        length*=cpu.data_regs[1];                          // nbr of megabytes → bytes          
        if (this.nativeFileNames[drive]==null || length/512>maax)
        {
            cpu.data_regs[0]=smsqmulator.Types.ERR_FMTF;   // no filename
            return;                                             
        }
        try
        {
            if ( this.raFile[drive]!=null)
            {
                this.raFile[drive].close();
            }
            this.raFile[drive] = new java.io.RandomAccessFile(this.nativeFileNames[drive],"rw");// read/write access to this file
            this.ioChannel[drive] = this.raFile[drive].getChannel();// and the associated file channel
        }
        catch (Exception e)
        {
            cpu.data_regs[0]=smsqmulator.Types.ERR_FMTF;
            return;
        }
        // as of here, file exists and channel is open to it.
        try
        {
            java.nio.ByteBuffer buf = java.nio.ByteBuffer.allocate(512);  
            while (this.ioChannel[drive].position()<length)
            {
                buf.position(0);
                this.ioChannel[drive].write(buf);                   // extend file to have required length and make sure all sectors are 0
            }
            this.ioChannel[drive].force(true);
            cpu.data_regs[1]=cpu.data_regs[2]=(int) (length/512);
            cpu.data_regs[0]=0;
        }
        catch (Exception e)
        {
            cpu.data_regs[0]=smsqmulator.Types.ERR_FMTF;   
        }
    }
    
    
    /**
     * Attempts to lock the drive.
     * 
     * Error or succes return code in D0.
     * 
     * @param drive which drive to lock (0...7)
     */
    private void lockDrive(int drive,CPU.MC68000Cpu cpu)
    {
        if (this.raFile[drive]==null)
            checkAndOpenDrive(drive,cpu);
        try
        {
            if (!this.readOnly[drive] && (!(this.flock[drive]!=null && this.flock[drive].isValid())))
            {
                this.flock[drive]=this.ioChannel[drive].tryLock(); // try to lock this file
                if (this.flock[drive]==null || !this.flock[drive].isValid() || this.flock[drive].isShared())
                {        
                    this.flock[drive]=null;
                    cpu.data_regs[0]=smsqmulator.Types.ERR_MCHK;// couldn't get a valid lock, forget it
                }
            }
        }
        catch (Exception e)
        {      
            this.flock[drive]=null;
            cpu.data_regs[0]=smsqmulator.Types.ERR_MCHK;
        }
    }  
        
    /**
     * Sets the names of the native directories to be used for each drive of the device.
     * Checks that a file is a valid QL floppy image file.
     * <p> In keeping with standard SMSQE practice, it is presumed throughout that each device may have 8 drives.
     *
     * @param names an 8 element String array with names for the drives.
     * 
     * @return <code>true</code> if names were set, else <code>false</code>.
     */
    public boolean setNames(String[]names)
    {
        java.util.ArrayList<String> errStr=new java.util.ArrayList<>();
        for (int i= 0;i<8;i++)
        {
            if (names[i]!= null)
            {
                String s = names[i].trim();
                if (s.isEmpty())
                    this.nativeFileNames[i]=null;
                else
                    this.nativeFileNames[i]=s;
            }
            else
                this.nativeFileNames[i]=null;
        }
        return true;
    }
    
    /**
     * Checks whether a native file seems to be a valid QL disk image file, and opens the RA file to it and sets readOnly status.
     * It does not try to lock the file.
     * Note that it ALWAYS closes an existing RA file first.
     * 
     * @param driveNbr which drive we're trying to check (0...7, presumed to be correct).
     */
    private void checkAndOpenDrive(int driveNbr,CPU.MC68000Cpu cpu) 
    {
        if (this.raFile[driveNbr]!=null)
        {
            cpu.data_regs[0]=0;
            return;
        }
        cpu.data_regs[0]=smsqmulator.Types.ERR_FDNF;       // presume error
        if (this.nativeFileNames[driveNbr]==null)
        {
            return;
        }
        java.io.File device=new java.io.File(this.nativeFileNames[driveNbr]);    // the drive were talking about
        if (!device.exists())
        {
            return;
        }
        this.readOnly[driveNbr]=!device.canWrite();
        cpu.data_regs[0]=smsqmulator.Types.ERR_MCHK;       // preset error
        try
        {
            if (this.raFile[driveNbr]!=null)
                this.raFile[driveNbr].close(); 
            if (this.readOnly[driveNbr])
                this.raFile[driveNbr]=new java.io.RandomAccessFile(device,"r");// read  access to this file
            else
                this.raFile[driveNbr]=new java.io.RandomAccessFile(device,"rw");// read/write access to this file

            this.ioChannel[driveNbr] = this.raFile[driveNbr].getChannel();// and the associated file channel
            java.nio.ByteBuffer driveMap=java.nio.ByteBuffer.allocate(512);
            int bytesRead=this.ioChannel[driveNbr].read(driveMap);     // read the first sector (512 bytes - NOT the first cluster) of the drive
            boolean badRead=(bytesRead!=512 || driveMap.getInt(0)!=WinDriver.QLWA);// I must have read exactly this amount and marker must check
            if (badRead)
            {
                closeRAFile(driveNbr);
                return;                                         // this can't be a true QLWx file
            }
            if (!this.readOnly[driveNbr])
            {
                int update= driveMap.getInt(drivers.win.WinDrive.QWA_UCHK );
                driveMap.putInt(drivers.win.WinDrive.QWA_UCHK,update+1);
                this.ioChannel[driveNbr].write(driveMap, 0);
                this.ioChannel[driveNbr].position(0);
            }
        }
        catch (Exception e)
        {
            return;
        }
      //  cpu.writeMemoryByte(cpu.addr_regs[4]+0x24, 1);// sf ddf_mstat(a4)
        cpu.data_regs[0]=0;        // if we get here, all is ok, but signal new drives
    }
    
   /**
     * CLoses the random access file, ignoring all close errors.
     * 
     * @param driveNbr drive for which the access file is to be closed (0...7, presumed to be correct)
     */
    public void closeRAFile(int driveNbr)
    {
        if (this.raFile[driveNbr]!=null)
        {
            try
            {
                this.raFile[driveNbr].close();                             // if I haven't, this cannot be a qxl.win file.
                this.flock[driveNbr]=null;
            }
            catch (java.io.IOException e)
            {
                /*nop*/
            }
        }
        this.raFile[driveNbr]=null;
    }
    
    /**
     * Gets the names of the native files used for each drive of the device.
     * <p> In keeping with standard SMSQE practice, it is presumed throughout that each device may have 8 drives.
     *
     * @param deviceID an int containing the name of the device for which the names are to be returned (eg 'NFA0', i.e. 0x0x4e464130).
     * 
     * @return An 8 elements string array with the names, or <code>null</code> if the device ID wasn't mine. 
     * Individual elements may be <code>null</code> or empty if the corresponding drive isn't assigned.
     */
    public String[] getNames(int deviceID) 
    {
        if (smsqmulator.Types.FLPDriver!=deviceID)
            return null;
        return this.nativeFileNames.clone();
    }
    
    /**
     * This sets the usage name of the device, eg. "FLP_USE MDV"
     * 
     * @param deviceID the true name of the device as an int.
     * @param usage the usage name as an int
     * @return true if set ok, else false;
     */
    public boolean setUsage(int deviceID,int usage) 
    { 
        if (smsqmulator.Types.FLPDriver==deviceID)
        {
            this.usage=usage;
            return true;
        }
        return false;
    }
    
    /**
     * This gets the usage name of the device.
     * 
     * @param deviceID the true name of the device as an int.
     * 
     * @return the usage name as an int, or 0 if this wasn't for this device
     */
    public int getUsage(int deviceID)
    {
        if (deviceID==smsqmulator.Types.FLPDriver)
            return this.usage;
        return 0;
    }
   
    /**
     * Gets the device ID for this device, eg. SFA0.
     * 
     * @return the device ID as int.
     */
    public int getDeviceID()
    {
        return smsqmulator.Types.FLPDriver;
    }
    
    /**
     * Sets the cpu used by the device driver.
     * 
     * @param cpu the cpu to be set.
     */
    public void setCpu (CPU.MC68000Cpu cpu)
    {
        cpu=cpu;
    }
    
    /**
     * Sets the name of a native directory, as passed in a datastructure pointed to by SMSQE.
     * That data is also saved in the inifile.
     * The pointer to the name lies at 4(a1).
     * 
     * @param driveNbr the drive (from 0..7 , presumed correct) for which the name is to be obtained
     * 
     */
    public void setName(int driveNbr,CPU.MC68000Cpu cpu)
    {
        int addr = cpu.readMemoryLong(cpu.addr_regs[1]+4);
        String fileName = cpu.readSmsqeString(addr).trim();// address is pointed to by (A1).
        if (this.raFile[driveNbr]!=null)
            closeRAFile(driveNbr);
        if (fileName.isEmpty())
            fileName=null;
        this.nativeFileNames[driveNbr] = fileName;
        cpu.data_regs[0]=0;
        int A4 = cpu.addr_regs[4];
        if (A4!=0)
        {
            addr=cpu.readMemoryLong(0x400)+0x100;              // ptr to sysvars drv defn block in syvars
            int stop = addr+64;
            while(addr<stop)
            {
                if (cpu.readMemoryLong(addr)==A4)
                {
                    cpu.writeMemoryLong(addr, 0);
                    break;
                }
                addr+=4;
            }
        }
    }
    
    /**
     * Gets the name of the native directory used for a drive and puts it into a buffer within SMSQE.
     * The buffer is pointed to by 4(A1) and contains, in its first word, its length.
     * 
     * @param driveNbr the drive (from 0..7 , presumed correct) for which the name is to be obtained
     * 
     */
    public void getName (int driveNbr,CPU.MC68000Cpu cpu)
    {
        int bufferAddress = cpu.readMemoryLong(cpu.addr_regs[1]+4);
        int bufferSize = cpu.readMemoryWord(cpu.addr_regs[1]+2);
        if (this.nativeFileNames[driveNbr]==null)
            cpu.writeMemoryWord(bufferAddress,0);                  // there is nothing there
        else
        {
            cpu.writeSmsqeString(bufferAddress, this.nativeFileNames[driveNbr], true, bufferSize);
        }
        if (bufferSize<this.nativeFileNames.length)
             cpu.data_regs[0]=smsqmulator.Types.ERR_BFFL;
        else
             cpu.data_regs[0]=0;
    }
    
    public void reset()
    {
        
    }

    private void setRemovable(int driveNbr, boolean state)
    {
        this.isRemovable[driveNbr]=state;
    }      
    
    private int getRemovable(int driveNbr)
    {
        return this.isRemovable[driveNbr]?1:0;
    }    

}