package smsqmulator;

/**
 * A thread to monitor changes from the clipboard, by comparing current content with old content.
 * 
 * @version  1.00
 * 
 * @author and copyright (c) wolfgang Lenerz 2017-2020
 * 
 */
public class ClipboardMonitorThread extends Thread
{
    private final ClipboardXfer clipboardXfer;                  // clibboard object to get text from
    private String old = null;                                  // text gotten from the previous call to the clipboard
    private volatile boolean isCancelled = false;
    
    /**
     * Create this object.
     * 
     * @param cbxfer the object which handles transfer
     * @param m String already in the clipboard when this is created.
     */
    public ClipboardMonitorThread(ClipboardXfer cbxfer,String m)
    {
        this.clipboardXfer=cbxfer;
        if (m!=null)
            this.old=m.toLowerCase();
    }

    /**
     * Stop this thread.
     */
    public void cancel()
    {
        this.isCancelled=true;
    }
    
    @Override
    /**
     * This checks every half second if the text in the clipboard has changed.
     */
    public void run() 
    {
        while (!this.isCancelled)
        {
            try 
            {
                this.sleep(500);
                
                String current = this.clipboardXfer.getClipboardContents();  // get clipboard content, guaranteed not to be NULL.
                if (current.isEmpty()) 
                    continue;                                   // none
                String lc=current.toLowerCase();
                if (this.old == null || !(this.old.equals(lc)))
                {
                    this.clipboardXfer.setNewContent(current);  // yes,yes that is correct, even though we just got that content from the clipboard
                    this.old=lc;
                }
            }
            catch (InterruptedException e) 
            {
                return;                 
            }      
        }
    }
}
   