
package smsqmulator;

import screens.Screen16;
import screens.Screen32;
import screens.Screen0;
import screens.Screen;
import java.io.InvalidClassException;

/**
 * A class to generate, save and load snapshots.
 * The format of a snapshot is
 *  - General header
 *  - memory header
 *  - memory (array of shorts)
 *  - Screen header
 *  - Registers header
 *  - Registers d0-d7   (8 ints)
 *  - Registers a0-a7   (8 ints)
 *  - SP                (1 int)
 *  - status reg        (1 int)
 *  - Memory header
 *  - amount of memory  (1 int)
 *  - ... xxx bytes of memory
 *  - Screen header
 *  - screen mode       (1 int)
 *  - amount of screen memory
 *  - ... xxx bytes screen memory
 * -
 * 
 * @author and copyright (c) wolfgang lenerz 2018
 */
public class Snapshot
{
    private static final String GENERAL_HEADER = "SMSQmulator Snapshot file";
    private static final String REGISTERS_HEADER = "SMSQmulator Snapshot file REGISTERS";
    private static final String MEMORY_HEADER = "SMSQmulator Snapshot file MEMORY";
    private static final String VIDEO_HEADER = "SMSQmulator Snapshot file VIDEO";
    private static final String QXLWIN_HEADER = "SMSQmulator Snapshot file WIN drive names";
    private static final String NFA_HEADER = "SMSQmulator Snapshot file NFA drive names";
    private static final String SFA_HEADER = "SMSQmulator Snapshot file SFA drive names";
    private static final String FLP_HEADER = "SMSQmulator Snapshot file FLP drive names";
 
  //  private static final String VIDEO_HEADER = "SMSQmulator Snapshot file VIDEO";
 
    //private static final String VIDEO_HEADER = "SMSQmulator Snapshot file VIDEO";
 
    public static void writeSnapshot(CPU.MC68000Cpu cpu, TrapDispatcher td, String filename)
    {
        try
        (
            java.io.FileOutputStream fos = new java.io.FileOutputStream(filename);
            java.io.ObjectOutputStream oos = new java.io.ObjectOutputStream(fos))
        {
            
            oos.writeObject(Snapshot.GENERAL_HEADER);
            
            // Memory
            
            oos.writeObject(Snapshot.MEMORY_HEADER);
            short[] mainMem=cpu.getMemory();
            oos.writeObject(mainMem);
            
            // Screen
            oos.writeObject(Snapshot.VIDEO_HEADER);
            Screen screen = cpu.getScreen();
            oos.writeObject("Screen type :");
            if (screen instanceof Screen0)
                oos.writeObject("Screen0");
            else if (screen instanceof Screen16)
                oos.writeObject("Screen16");
            else if (screen instanceof Screen32)
                oos.writeObject("Screen32");
            else
                throw new java.io.InvalidClassException ("Inexistent screen mode");
            //     oos.writeObject("Screen size :");
            //   oos.writeObject(screen.getImageSize());         // dimension
            oos.writeObject("Screen doubled ?");
            oos.writeObject(screen.isDouble);
            
            
            oos.writeObject(Snapshot.QXLWIN_HEADER);
            String []temp=td.getNamesForDrives(Types.WINDriver);
            oos.writeObject(temp);
            oos.writeObject(Snapshot.FLP_HEADER);
            temp=td.getNamesForDrives(Types.FLPDriver);
            oos.writeObject(temp);
            oos.writeObject(Snapshot.NFA_HEADER);
            temp=td.getNamesForDrives(Types.NFADriver);
            oos.writeObject(temp);
            oos.writeObject(Snapshot.SFA_HEADER);
            temp=td.getNamesForDrives(Types.SFADriver);
            oos.writeObject(temp);
            
            // Registers
            oos.writeObject(Snapshot.REGISTERS_HEADER);
            oos.writeObject(cpu.data_regs);
            oos.writeObject(cpu.addr_regs);
            oos.writeObject(cpu.pc_reg);
            oos.writeObject(cpu.reg_sr);
            oos.writeObject(cpu.reg_usp);
            oos.writeObject(cpu.reg_ssp);
        
           // oos.writeObject(Snapshot.VIDEO_HEADER);
           // oos.writeObject(screen);
            
            oos.writeObject(Snapshot.REGISTERS_HEADER);
            
            oos.close();
            
            /*
               oos.writeObject(screen);
            oos.writeObject(screen);
            oos.writeObject(screen);
            oos.writeObject(screen);
            
            oos.writeObject("Screen data");
            oos.writeObject(screen);
            oos.writeObject("Memory :");
         */
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    
    
    /**
     * Reads a snapshot.
     * 
     * @param cpu
     * @param td
     * @param filename 
     */
    public static void readSnapshot(CPU.MC68000Cpu cpu, TrapDispatcher td, String filename)
    {
        try
        (
            java.io.FileInputStream fos = new java.io.FileInputStream(filename);
            java.io.ObjectInputStream oos = new java.io.ObjectInputStream(fos))
        {
            String temp;
            Screen tempScreen;
            String[]flp=null;
            String[]qxl=null;
            String[]nfa=null;
            String[]sfa=null;
            Object tempObj=oos.readObject();
            checkObject (tempObj,"String",Snapshot.GENERAL_HEADER,"General header");
            
            // Memory
            
            tempObj=oos.readObject();
            checkObject (tempObj,"String",Snapshot.MEMORY_HEADER,"Memory header"); // check header
            tempObj = oos.readObject();                     // mem array
            checkObject (tempObj,"Short array","","Main memory array"); // check
            short[] memArray=(short[])tempObj;
            
            // Screen
            tempObj=oos.readObject();
            checkObject (tempObj,"String",Snapshot.VIDEO_HEADER,"Screen header"); // check header
            tempObj=oos.readObject();
            checkObject (tempObj,"String","Screen type :","Screen type header"); // check header
       
            tempObj=oos.readObject();
            checkObject (tempObj,"String","","Screen type :"); // check header
            temp= (String) tempObj;
        /*
            
            switch (temp)
            {
                case "Screen0":
                    tempObj=oos.readObject();
                    checkObject (tempObj,"Screen0","","Screen 0"); // check header
                    break; 
                case "Screen16":
                    tempObj=oos.readObject();
                    checkObject (tempObj,"Screen16","","Screen 16"); // check header
                    break;
                case "Screen32":
                    tempObj=oos.readObject();
                    checkObject (tempObj,"Screen32","","Screen 32"); // check header
                    break;
                default:
                    throw new java.io.InvalidClassException ("Inexistent screen mode");
            }
            
            */
            tempObj=oos.readObject();
            checkObject (tempObj,"String", "Screen doubled ?", "Screen doubled header"); // check header
            tempObj=oos.readObject();
            checkObject (tempObj,"Boolean", "", "Screen doubled value"); // check header
            Boolean b=(Boolean) tempObj;
          
            // Native drive names
            tempObj=oos.readObject();
            checkObject (tempObj,"String",Snapshot.QXLWIN_HEADER,"qxlwin header"); // check header
            tempObj=oos.readObject();
            if (tempObj!=null)
            {
                checkObject (tempObj,"String array","","qxlwin content"); 
                qxl=(String[]) tempObj;
            }
            tempObj=oos.readObject();
            checkObject (tempObj,"String",Snapshot.FLP_HEADER,"flp header"); // check header
            tempObj=oos.readObject();
            if (tempObj!=null)
            {
                checkObject (tempObj,"String array","","flp content");
                flp=(String[]) tempObj;
            }
            tempObj=oos.readObject();
            checkObject (tempObj,"String",Snapshot.NFA_HEADER,"nfa header"); // check header
            tempObj=oos.readObject(); 
            if (tempObj!=null)
            {
                checkObject (tempObj,"String array","","nfa content");
                nfa=(String[]) tempObj;
            }
            tempObj=oos.readObject(); 
            checkObject (tempObj,"String",Snapshot.SFA_HEADER,"sfa header"); // check header
            tempObj=oos.readObject();
            if (tempObj!=null)
            {
                checkObject (tempObj,"String array","","sfa content"); 
                sfa=(String[]) tempObj;
            }
          
            // Registers tempObj=oos.readObject();
            tempObj=oos.readObject();
            checkObject (tempObj,"String",Snapshot.REGISTERS_HEADER,"Registers header"); // check header
            tempObj=oos.readObject();
            checkObject (tempObj,"Int array","","Data registers"); // check header
            int [] dataregs=(int [])tempObj;
            tempObj=oos.readObject();
            checkObject (tempObj,"Int array","","Address registers"); // check header
            int [] addregs=(int [])tempObj;
            tempObj=oos.readObject();
            checkObject (tempObj,"Int","","PC register"); // check header
            int pcreg=(int)tempObj;
            tempObj=oos.readObject();
            checkObject (tempObj,"Int","","Status register"); // check header
            int srreg=(int)tempObj;
            tempObj=oos.readObject();
            checkObject (tempObj,"Int","","USP"); // check header
            int usp=(int)tempObj;
            tempObj=oos.readObject();
            checkObject (tempObj,"Int","","SSP"); // check header
            int ssp=(int)tempObj;
           
            oos.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    
    /**
     * 
     * @param obj the object to check.
     * @param type the type this object should have
     * @param content the content this object should have (e.g the valueof a string).
     * @param name the name of the object
     * @throws InvalidClassException 
     */
    private static final void checkObject(Object obj,String type, Object content,String name) throws InvalidClassException
    {
        String objContent=null;
        switch (type)
        {
            case "String":
                if (!(obj instanceof java.lang.String))
                    throw new java.lang.AssertionError(" Object "+name+" is not String");
                else
                    objContent = (String)obj;
                String temp = (String)content;
                if (temp!=null && !temp.isEmpty())
                {
                    if (!(temp.equals(objContent)))
                        throw new java.lang.AssertionError("String " + objContent + " does not maatch obligatory content : "+temp);
                }
                break;
                
            case "Short array":
                if (!(obj instanceof short[]))
                    throw new java.lang.AssertionError("Object "+name+" is not an array of short");
                break;
            
            case "String array":
                if (!(obj instanceof java.lang.String[]))
                    throw new java.lang.AssertionError("Object "+name+" is not an array of string");
                break;
                  
            case "Int array":
                if (!(obj instanceof int[]))
                    throw new java.lang.AssertionError("Object "+name+" is not an array of string");
                break;
                
            case "Screen 0":
                if (!(obj instanceof Screen0))
                    throw new java.lang.AssertionError("Object "+name+" is not a Screen 0");
                break;
                
            case "Screen 16":
                if (!(obj instanceof Screen16))
                    throw new java.lang.AssertionError("Object "+name+" is not a Screen 16");
                break;
                
            case "Screen 32":
                if (!(obj instanceof Screen32))
                    throw new java.lang.AssertionError("Object "+name+" is not a Screen 32");
                break;
                
            case "Dimension":
                if (!(obj instanceof java.awt.Dimension))
                    throw new java.lang.AssertionError("Object "+name+" is not a Dimension");
                break;
  
            case "Boolean":
                if (!(obj instanceof Boolean))
                    throw new java.lang.AssertionError("Object "+name+" is not a Boolea");
                break;
                
            case "Int":
                if (!(obj instanceof Integer))
                    throw new java.lang.AssertionError("Object "+name+" is not an int");
                break;
                
            default:
                throw new java.io.InvalidClassException (name +" - Inexistent class :\n"+ obj);
        }
    }
}
